@@ -13,7 +13,8 @@ char get_header_tar_bz2(archive_handle_t *archive_handle)
 
 	archive_handle->src_fd = open_transformer(archive_handle->src_fd, unpack_bz2_stream, "bunzip2", "bunzip2", "-cf", "-", NULL);
 	archive_handle->offset = 0;
-	while (get_header_tar(archive_handle) == EXIT_SUCCESS) /**/;
+	while (get_header_tar(archive_handle) == EXIT_SUCCESS)
+		continue;
 
 	/* Can only do one file at a time */
 	return EXIT_FAILURE;
