@@ -25,7 +25,7 @@ struct kbentry {
 #define NR_KEYS         128
 #define MAX_NR_KEYMAPS  256
 
-int loadkmap_main(int argc, char **argv);
+int loadkmap_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int loadkmap_main(int argc, char **argv)
 {
 	struct kbentry ke;
