@@ -16,24 +16,32 @@
 static void watchdog_shutdown(int ATTRIBUTE_UNUSED sig) ATTRIBUTE_NORETURN;
 static void watchdog_shutdown(int ATTRIBUTE_UNUSED sig)
 {
-	write(3, "V", 1);	/* Magic, see watchdog-api.txt in kernel */
+	static const char V = 'V';
+
+	write(3, &V, 1);	/* Magic, see watchdog-api.txt in kernel */
 	if (ENABLE_FEATURE_CLEAN_UP)
 		close(3);
 	exit(0);
 }
 
-int watchdog_main(int argc, char **argv);
+int watchdog_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int watchdog_main(int argc, char **argv)
 {
 	unsigned opts;
-	unsigned timer_duration = 30; /* Userspace timer duration, in seconds */
+	unsigned timer_duration = 30000; /* Userspace timer duration, in milliseconds */
 	char *t_arg;
 
 	opt_complementary = "=1"; /* must have 1 argument */
 	opts = getopt32(argv, "Ft:", &t_arg);
 
-	if (opts & OPT_TIMER)
-		timer_duration = xatou(t_arg);
+	if (opts & OPT_TIMER) {
+		static const struct suffix_mult suffixes[] = {
+			{ "ms", 1 },
+			{ "", 1000 },
+			{ }
+		};
+		timer_duration = xatou_sfx(t_arg, suffixes);
+	}
 
 	if (!(opts & OPT_FOREGROUND)) {
 		bb_daemonize_or_rexec(DAEMON_CHDIR_ROOT, argv);
@@ -50,10 +58,8 @@ int watchdog_main(int argc, char **argv)
 		 * Make sure we clear the counter before sleeping, as the counter value
 		 * is undefined at this point -- PFM
 		 */
-		write(3, "", 1);
-		sleep(timer_duration);
+		write(3, "", 1); /* write zero byte */
+		usleep(timer_duration * 1000L);
 	}
-
-	watchdog_shutdown(0);
-	/* return EXIT_SUCCESS; */
+	return EXIT_SUCCESS; /* - not reached, but gcc 4.2.1 is too dumb! */
 }
