@@ -241,12 +241,12 @@ static void parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
 		} else if (key == ARG_remote) {
 			NEXT_ARG();
 			key = index_in_strings(keywords, *argv);
-			if (key == ARG_any)
+			if (key != ARG_any)
 				p->iph.daddr = get_addr32(*argv);
 		} else if (key == ARG_local) {
 			NEXT_ARG();
 			key = index_in_strings(keywords, *argv);
-			if (key == ARG_any)
+			if (key != ARG_any)
 				p->iph.saddr = get_addr32(*argv);
 		} else if (key == ARG_dev) {
 			NEXT_ARG();
@@ -482,7 +482,7 @@ static void do_tunnels_list(struct ip_tunnel_parm *p)
 		    (p->i_key && p1.i_key != p->i_key))
 			continue;
 		print_tunnel(&p1);
-		puts("");
+		bb_putchar('\n');
 	}
 }
 
@@ -512,7 +512,7 @@ static int do_show(int argc, char **argv)
 		return -1;
 
 	print_tunnel(&p);
-	puts("");
+	bb_putchar('\n');
 	return 0;
 }
 
